#include "DMatrixBasic.h"
#include "TokenFile.h"
#include <fstream.h>
#include <stdlib.h>
#include <Afxtempl.h>

/////////////////////////////////////////////////////////////////////////////
//	sqaure each element
//
//	author:	Rune Fisker, 1/5-1999
void CDVectorBasic::Sqr()
{

	for (int i=0;i < Length(); i++)
	{
		(*this)[i] *= (*this)[i];
	}
}

/////////////////////////////////////////////////////////////////////////////
//	take the sqaure root of each element
//
//	author:	Rune Fisker, 1/5-1999
void CDVectorBasic::Sqrt()
{

	for (int i=0;i < Length(); i++)
	{
		(*this)[i] = sqrt((*this)[i]);
	}
}

/////////////////////////////////////////////////////////////////////////////
//	take the power of each pixel
//
//	input:
//		dP:	power
//
//	author:	Rune Fisker, 1/5-1999
void CDVectorBasic::Pow(double dP)
{
	for (int i=0;i < Length(); i++)
	{
		(*this)[i] = pow((*this)[i],dP);
	}
}

/////////////////////////////////////////////////////////////////////////////
//	multiply each element in the matrix with the corresponding element of the input matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDVectorBasic::ElementMultiply(const CDVectorBasic& vector)
{
	assert(Length() == vector.Length());

	for (int i=0;i < Length(); i++)
	{
		(*this)[i] *= vector[i];
	}
}

/////////////////////////////////////////////////////////////////////////////
//	divide each element in the matrix with the corresponding element of the input matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDVectorBasic::ElementDivide(const CDVectorBasic& vector)
{
	assert(Length() == vector.Length());

	for (int i=0;i < Length(); i++)
	{
		(*this)[i] /= vector[i];
	}
}

/////////////////////////////////////////////////////////////////////////////
//	return the iRth row of the matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::Row(int iR, CDVectorBasic& vRow)
{
	assert(NCols() == vRow.Length());

	for (int i=0;i<NCols();i++)
	{
		vRow[i] = (*this)[iR][i];
	}
}

/////////////////////////////////////////////////////////////////////////////
//	return the iRth column of the matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::Col(int iC, CDVectorBasic& vCol)
{
	assert(NRows() == vCol.Length());

	for (int i=0;i<NRows();i++)
	{
		vCol[i] = (*this)[i][iC];
	}
}

/////////////////////////////////////////////////////////////////////////////
//	set the iRth row of the matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::SetRow(int iR,const CDVectorBasic& vRow)
{
	assert(NCols() == vRow.Length());

	for (int i=0;i<NCols();i++)
	{
		(*this)[iR][i] = vRow[i];
	}
};

/////////////////////////////////////////////////////////////////////////////
//	set the iRth column of the matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::SetCol(int iC,const CDVectorBasic& vCol)
{
	assert(NRows() == vCol.Length());

	for (int i=0;i<NRows();i++)
	{
		(*this)[i][iC] = vCol[i];
	}

};

/////////////////////////////////////////////////////////////////////////////
//  This function returns a string representing the matrix. 
//
//	author:	Rune Fisker, 1/5-1999
void CDVectorBasic::ToString(CString& strOut)
{
	strOut = "";

	CString strTmp;
	
	for(int i=0;i<Length();i++)
	{
		strTmp.Format("%8.3f ",(*this)[i]);
		strOut += strTmp;
	}
	strOut += "\r\n";	
}

//Calculates the 2-norm of a vector.
double CDVectorBasic::Norm2()
{
	double norm=0;

	for(int c=0;c<(*this).Length();c++)
	{
		norm+=(*this)[c]*(*this)[c];
	}
	return sqrt(norm);

}

// Normalize the vector to unit length, using the 2-norm.
void CDVectorBasic::Normalize2()
{
	double norm;

	norm=(*this).Norm2();

	for(int c=0;c<(*this).Length();c++)
	{
		(*this)[c]/=norm;;
	}

}

// Saves the Vector as an MatLAb m-file.
void CDVectorBasic::MFile(const char *szFilename)
{
	CString		sToStr;
	ofstream	fstreamOut(szFilename);

	ToString(sToStr);

	fstreamOut	<<	"Vec=[";
	fstreamOut	<<	sToStr;
	fstreamOut	<<	"];";

}



//	This function transforms the matrix into a diagonal matrix,
//	with the values of vec in the diagonal.

void CDMatrixBasic::Diag(const CDVectorBasic &vec)
{
	int stop=min(min(NRows(),NCols()),vec.Length() ); //The maximum amount of values it is posible to transfere.
	*this = 0.0;
	for(int i=0;i<stop;i++)
	{
		(*this)[i][i]=vec[i];
	}
}



//  This function returns a string representing the matrix. 
void CDMatrixBasic::ToString(CString& strOut)
{
	strOut = "";

	CString strTmp;

	for(int i=0;i<NRows();i++)
	{
		for(int j=0;j<NCols();j++)
		{
			strTmp.Format("%8.3f ",(*this)[i][j]);
			strOut += strTmp;
		}
		strOut += "\r\n";
	}
}

//Coposes the matrix of Top and Bottom Verticaly.
void CDMatrixBasic::CombVert(CVisDMatrix &Top, CVisDMatrix &Bottom)
{
	assert(Top.NCols()==Bottom.NCols());
	assert(Top.NCols()==(*this).NCols());
	assert(Top.NRows()+Bottom.NRows()==(*this).NRows());


	int j,i;

	for(j=0;j<Top.NRows();j++)
	{
		for(i=0;i<(*this).NCols();i++)
		{
			(*this)[j][i]=Top[j][i];
		}
	}
	for(;j<(*this).NRows();j++)
	{
		for(i=0;i<(*this).NCols();i++)
		{
			(*this)[j][i]=Bottom[j-Top.NRows()][i];
		}
	}
}

//Returns the sub matrix indicated by the coordinates, start coordinat points included. 
void CDMatrixBasic::SubOf(CVisDMatrix M, int nStartRow, int nEndRow, int nStartCol, int nEndCol)
{

	assert( nStartRow<M.NRows() && nStartCol<M.NCols() );
	assert( nEndRow<=M.NRows() && nEndCol<=M.NCols() );
	assert(	-1<nStartRow && -1<nStartCol );
	assert( (*this).NRows() <= nEndRow-nStartRow );
	assert( (*this).NCols() <= nEndCol-nStartCol );


	for(int j=nStartRow;j<nEndRow;j++)
	{
		for(int i=nStartCol;i<nEndCol;i++)
		{
			(*this)[j-nStartRow][i-nStartCol]=M[j][i];
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//	Writes the contex of the matrix to MatLab and denotes it by 'sName'.
//
//	author:	Henrik Aans, 14/7-1999
void CDMatrixBasic::ToMatlab(const CString& sFilename,
							 const CString& sName,
							 const CString& sComment,
							 bool fAppend)
{
	CString		strTmp;
	if(fAppend)
	{
		ofstream	fstreamOut(sFilename,ios::app);	//open stream

		if(sComment!="")
		{
			fstreamOut	<<	"%";
			fstreamOut	<<	sComment;
			fstreamOut	<<	"\n";

		}

		fstreamOut	<<	sName;						//Write the name of the matrix.
		fstreamOut	<<	"=[\n";

		//Writes the matrix contents to sToStr.
		for(int i=0;i<NRows();i++)
		{
			for(int j=0;j<NCols();j++)
			{
				strTmp.Format("%8.6f ",(*this)[i][j]);
				fstreamOut	<<  strTmp;
			}
			fstreamOut	<<	";\n";
		}
		fstreamOut	<<	"];\n"; 
	}
	else
	{
		ofstream	fstreamOut(sFilename,ios::trunc);	//open stream

		if(sComment!="")
		{
			fstreamOut	<<	"%";
			fstreamOut	<<	sComment;
			fstreamOut	<<	"\n";

		}

		
		fstreamOut	<<	sName;						//Write the name of the matrix.
		fstreamOut	<<	"=[\n";

		//Writes the matrix contents to sToStr.
		for(int i=0;i<NRows();i++)
		{
			for(int j=0;j<NCols();j++)
			{
				strTmp.Format("%8.6f ",(*this)[i][j]);
				fstreamOut	<<  strTmp;
			}
			fstreamOut	<<	";\n";
		}
		fstreamOut	<<	"];\n"; 
	}

}

/////////////////////////////////////////////////////////////////////////////
//	take the sqaure root of each element
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::Sqrt()
{
	for(int j=0;j<(*this).NRows();j++)
	{
		for(int i=0;i<(*this).NCols();i++)
		{
			(*this)[j][i] = sqrt((*this)[j][i]);
		}
	}

}


/////////////////////////////////////////////////////////////////////////////
//	take the sqaure root of each element
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::Sqr()
{
	for (int r=0;r<NRows();r++)
	{
		for (int c=0;c<NCols();c++)
		{
			(*this)[r][c] *= (*this)[r][c];
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//	multiply each element in the matrix with the corresponding element of the input matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::ElementMultiply(const CDMatrixBasic& matrix)
{
	assert(NCols() == matrix.NCols());
	assert(NRows() == matrix.NRows());

	for (int r=0;r<NRows();r++)
	{
		for (int c=0;c<NCols();c++)
		{
			(*this)[r][c] *= matrix[r][c];
		}
	}
}

/////////////////////////////////////////////////////////////////////////////
//	divide each element in the matrix with the corresponding element of the input matrix
//
//	author:	Rune Fisker, 1/5-1999
void CDMatrixBasic::ElementDivide(const CDMatrixBasic& matrix)
{
	assert(NCols() == matrix.NCols());
	assert(NRows() == matrix.NRows());

	for (int r=0;r<NRows();r++)
	{
		for (int c=0;c<NCols();c++)
		{
			(*this)[r][c] /= matrix[r][c];
		}
	}
}




/////////////////////////////////////////////////////////////////////////////
//	Retrives data from MAtLab to the matrix given denoted by 'sName'.
//
//	author:	Henrik Aans, 14/7-1999
void CDMatrixBasic::FromMatlab(const CString& sFilename,const CString& sName)
{
	CTokenFile	DIVABuf;					//Buffer or stream to the file.
	CString		sTokenString;				//String for retriving tokings, or element entities from DIVAbuf.
	bool		bFound=false;				//Indicator for the soughtafter matrix has been found.
	CArray<double,double>	Entry;			//Container for the entries or numbers untile size om matrix is known.
	int			cCols=0;					//Number of columns in the matrix.
	int			cRows=0;					//Number of rows in the matrix.
	int			cEntry=0;					//Counter for wich entry is to be writen to the matrix.

	//Open stream/buffer.
	if(! DIVABuf.Open(sFilename,CFile::modeRead,NULL))
	{
		throw CVisError("The requested file does not exist.",eviserrorUnknown,"FromMatlab","DMatrixBasic.cpp.cpp", __LINE__);
	}


	//Find the sought after matrix or fail.
	do{
		if(!DIVABuf.NextToken(sTokenString))
		{
			throw CVisError("The requested variable is not in the requested file.",eviserrorUnknown,"FromMatlab","DMatrixBasic.cpp.cpp", __LINE__);
		}
	}while(sTokenString!=sName+"=[");


	//Parse the first row of the matrix, and there by retrive the number of columns.
	//Get a token from the stream.
	if(!DIVABuf.NextToken(sTokenString))
	{
		throw CVisError("Error reading from file.",eviserrorUnknown,"FromMatlab","DMatrixBasic.cpp.cpp", __LINE__);
	}
	while(sTokenString!=";")				//";" marks the end of a line.
	{
		Entry.Add(atof(LPCTSTR(sTokenString)));		
		//Get a token from the stream.
		if(!DIVABuf.NextToken(sTokenString))
		{
			throw CVisError("Error reading from file.",eviserrorUnknown,"FromMatlab","DMatrixBasic.cpp.cpp", __LINE__);
		}
		cCols++;
	}

	//Parse the rest of the matrix 
	while(sTokenString!="];")				//"];" marks the end of a matrix.
	{
		if(sTokenString==";")				//";" marks the end of a line.
		{
			cRows++;
		}
		else
		{
			Entry.Add(atof(LPCTSTR(sTokenString)));
		}
		//Get a token from the stream.
		if(!DIVABuf.NextToken(sTokenString))
		{
			throw CVisError("Error reading from file.",eviserrorUnknown,"FromMatlab","DMatrixBasic.cpp.cpp", __LINE__);
		}
	}

	//Adjust the size of the (output)matrix.
	Resize(cRows,cCols);

	//Transfere the values.
	for(int cI=0;cI<cRows;cI++)
	{
		for(int cJ=0;cJ<cCols;cJ++)
		{
			(*this)[cI][cJ]=Entry[cEntry];
			cEntry++;
		}
	}
}

// Comunication with MatLab routines.
void CDVectorBasic::ToMatlab(const CString& sFilename,const CString& sName,const CString& sComment,bool fAppend)
{
	CDMatrixBasic mOut(1,Length());

	memcpy(mOut[0],&((*this)[0]),sizeof(double)*Length());
	mOut.ToMatlab(sFilename,sName,sComment,fAppend);


}

void CDVectorBasic::FromMatlab(const CString& sFilename,const CString& sName)
{
	CDMatrixBasic mOut;
	mOut.FromMatlab(sFilename,sName);

	Resize(mOut.NCols());
	memcpy(&((*this)[0]),mOut[0],sizeof(double)*Length());
}