#if !defined(CD_VIS_MATRIX_BASIC)
#define CD_VIS_MATRIX_BASIC
#define MATLABTODIVABUFFER "C:\\Progra~1\\matlab\\bin\\DIVA.m"


#include "VisMatrix.h"

////////////////////////////////////////////////////////////////////////////
//  
//  CLASS:
//      CDVectorBasic
//  
////////////////////////////////////////////////////////////////////////////
class CDVectorBasic : public virtual CVisDVector
{
public:
    // Constructors (default destructor and copy constructor)
    CDVectorBasic(void):CVisDVector(){};
    CDVectorBasic(int length, double *storage = 0):CVisDVector(length,storage) {};
    CDVectorBasic(const CDVectorBasic &vec):CVisDVector(vec) {};

    // Assignment
    CDVectorBasic& operator=(const CDVectorBasic &vIn) {CVisDVector& vTmp = *this; vTmp = vIn; return *this; };
    CDVectorBasic& operator=(double value) {CVisDVector& vTmp = *this; vTmp = value; return *this; };

	// methods
	void MFile(const char*szFilename);

	void ToString(CString& strOut);

	void ElementMultiply(const CDVectorBasic& vector);
	void ElementDivide(const CDVectorBasic& vector);
		
	void Normalize2();
	double Norm2();

	void Pow(double dP);
	void Sqrt();
	void Sqr();

	// Comunication with MatLab routines.
	void ToMatlab(const CString& sFilename,const CString& sName,const CString& sComment="",bool fAppend=true);
	void FromMatlab(const CString& sFilename,const CString& sName);

};


////////////////////////////////////////////////////////////////////////////
//  
//  CLASS:
//      CDMatrixBasic
//  
////////////////////////////////////////////////////////////////////////////
class CDMatrixBasic : public virtual CVisDMatrix
{
public:
    // Constructors (default destructor and copy constructor)
    CDMatrixBasic(void):CVisDMatrix(){};
    CDMatrixBasic(int rows, int cols, double *storage = 0):CVisDMatrix(rows,cols,storage) {};
    CDMatrixBasic(const CDMatrixBasic &mat):CVisDMatrix(mat) {};

    // Assignment
    CDMatrixBasic& operator=(const CDMatrixBasic &mat) {CVisDMatrix& mTmp = *this; mTmp = mat; return *this; };;
    CDMatrixBasic& operator=(double value) {CVisDMatrix& mTmp = *this; mTmp = value; return *this; };

	// methods

	// matrix manipulation
	void Col(int iC, CDVectorBasic& vCol);
	void Row(int iR, CDVectorBasic& vRow);

	void SetCol(int iC,const CDVectorBasic& vCol);
	void SetRow(int iR,const CDVectorBasic& vRow);
	void SubOf(CVisDMatrix M,int nStartRow,int nEndRow, int nStratCol, int nEndCol);


	void CombVert(CVisDMatrix& Top,CVisDMatrix& Bottom);

	void Diag(const CDVectorBasic& vec);
	void ToString(CString& strOut);

	void ElementDivide(const CDMatrixBasic& matrix);
	void ElementMultiply(const CDMatrixBasic& matrix);

	// math functions
	void Sqr();
	void Sqrt();

	// Comunication with MatLab routines.
	void ToMatlab(const CString& sFilename,const CString& sName,const CString& sComment="",bool fAppend=true);
	void FromMatlab(const CString& sFilename,const CString& sName);



};


#endif // CD_VIS_MATRIX_BASIC